@@ -9,15 +9,15 @@ App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.16
+Version 1.20
 
 =cut
 
-our $VERSION = '1.16';
+our $VERSION = '1.20';
 
 use base 'Exporter';
 
-our @EXPORT = qw( filetypes is_filetype interesting_files );
+our @EXPORT = qw( filetypes filetypes_supported is_filetype interesting_files _candidate_files _thpppt );
 
 =head1 SYNOPSIS
 
@@ -42,6 +42,29 @@ sub is_filetype {
     return;
 }
 
+our %types;
+our %mappings = (
+    cc          => [qw( c h )],
+    javascript  => [qw( js )],
+    parrot      => [qw( pir pasm pmc ops pod )],
+    perl        => [qw( pl pm pod tt ttml t )],
+    php         => [qw( php phpt htm html )],
+    python      => [qw( py )],
+    ruby        => [qw( rb )],
+    shell       => [qw( sh bash csh ksh zsh )],
+    sql         => [qw( sql ctl )],
+    yaml        => [qw( yaml )],
+);
+
+sub _init_types {
+    while ( my ($type,$exts) = each %mappings ) {
+        for my $ext ( @$exts ) {
+            push( @{$types{$ext}}, $type );
+        }
+    }
+}
+
+
 =head2 filetypes( $filename )
 
 Returns a list of types that I<$filename> could be.  For example, a file
@@ -49,33 +72,10 @@ F<foo.pod> could be "perl" or "parrot".
 
 =cut
 
-our %types;
-
-sub _set_up_types {
-    while ( @_ ) {
-        my $type = shift;
-        my $exts = shift;
-
-        for my $ext ( @$exts ) {
-            push( @{$types{$ext}}, $type );
-        }
-    }
-}
-
 sub filetypes {
     my $filename = shift;
 
-    _set_up_types(
-        cc          => [qw( c h )],
-        perl        => [qw( pl pm pod tt ttml t )],
-        parrot      => [qw( pir pasm pmc ops pod )],
-        php         => [qw( php phpt htm html )],
-        python      => [qw( py )],
-        ruby        => [qw( rb )],
-        shell       => [qw( sh bash csh ksh zsh )],
-        sql         => [qw( sql ctl )],
-        javascript  => [qw( js )],
-    ) unless keys %types;
+    _init_types() unless keys %types;
 
     if ( $filename =~ /\.([^.]+)$/ ) {
         my $ref = $types{lc $1};
@@ -103,6 +103,16 @@ sub filetypes {
     return;
 }
 
+=head2 filetypes_supported()
+
+Returns a list of all the types that we can detect.
+
+=cut
+
+sub filetypes_supported {
+    return keys %mappings;
+}
+
 =head2 interesting_files( \&is_interesting, $should_descend, @starting points )
 
 Returns an iterator that walks directories starting with the items
@@ -110,7 +120,8 @@ in I<@starting_points>.  If I<$should_descend> is false, don't descend
 into subdirectories. Each file to see if it's interesting is passed to
 I<is_interesting>, which must return true.
 
-Stolen from Mark Jason Dominus' marvelous I<Higher Order Perl>, page 126.
+All file-finding in this module is adapted from Mark Jason Dominus'
+marvelous I<Higher Order Perl>, page 126.
 
 =cut
 
@@ -134,21 +145,52 @@ sub interesting_files {
     }; # iterator
 }
 
+=head2 _candidate_files( $dir )
+
+Pulls out the files/dirs that might be worth looking into in I<$dir>.
+If I<$dir> is the empty string, then search the current directory.
+This is different than explicitly passing in a ".", because that
+will get prepended to the path names.
+
+=cut
+
 our %ignore_dirs = map { ($_,1) } qw( . .. CVS RCS .svn _darcs blib );
 sub _candidate_files {
     my $dir = shift;
 
-    my $dh;
-    if ( !opendir $dh, $dir ) {
-        warn "ack: $dir: $!\n";
-        return;
-    }
+    my @newfiles;
 
-    my @newfiles = grep { !$ignore_dirs{$_} } readdir $dh;
-    @newfiles = map "$dir/$_", @newfiles;
+    # TODO Refactor out the redundancy
+    if ( $dir eq "" ) {
+        my $dh;
+        if ( !opendir $dh, "." ) {
+            warn "ack: in current directory: $!\n";
+            return;
+        }
+
+        @newfiles = grep { !$ignore_dirs{$_} } readdir $dh;
+    }
+    else {
+        return $dir unless -d $dir;
+
+        my $dh;
+        if ( !opendir $dh, $dir ) {
+            warn "ack: $dir: $!\n";
+            return;
+        }
+
+        @newfiles = grep { !$ignore_dirs{$_} } readdir $dh;
+        @newfiles = map "$dir/$_", @newfiles;
+    }
     return @newfiles;
 }
 
+sub _thpppt {
+    my $y = q{_   /|,\\'!.x',=(www)=,   U   };
+    $y =~ tr/,x!w/\nOo_/;
+    print "$y ack --$_[0]!\n" and exit 0;
+}
+
 =head1 AUTHOR
 
 Andy Lester, C<< <andy at petdance.com> >>
